-- wengwengweng

local lg = love.graphics
local la = love.audio
local lf = love.filesystem

local function is(path, t)
	return lf.getInfo(path) and lf.getInfo(path).type == t
end

local function makeFont(path)
	return function(size)
		return lg.newFont(path, size)
	end
end

local function makeSound(path)
	return function(t)
		return la.newSource(path, t or "static")
	end
end

local loaders = {

	txt = lf.read,
	png = lg.newImage,
	jpg = lg.newImage,
	glsl = lg.newShader,
	mp3 = makeSound,
	wav = makeSound,
	ogg = makeSound,
	ttf = makeFont,
	otf = makeFont,
	fnt = lg.newFont

}

return function(dir)

	local dir = dir or "assets"

	if (not is(dir, "directory")) then
		error(dir .. " is not a valid directory")
	end

	local function get(t, k)

		local path = (t._path .. "/" .. k):gsub("^/+", "")

		if (is(path, "directory")) then

			rawset(t, k, init(path))

			return t[k]

		else

			for extension, loader in pairs(loaders) do

				local file = path .. "." .. extension

				if (loader and is(file, "file")) then

					local asset = loader(file)

					rawset(t, k, asset)

					return asset

				end

			end

	    end

	    return rawget(t, k)

	end

	init = function(p)
		return setmetatable({ _path = p }, { __index = get })
	end

	return init(dir)

end
