-- wengwengweng

local col = {}

function getPolyNextLine(vertices, index)
	if (index >= #vertices) then
		return vertices[index], vertices[1]
	else
		return vertices[index], vertices[index + 1]
	end
end

function col.getPolyRect(v)

	local minX = v[1][1]
	local maxX = v[1][1]
	local minY = v[1][2]
	local maxY = v[1][2]

	for i = 2, #v do

		if (v[i][1] < minX) then
			minX = v[i][1]
		elseif (v[i][1] > maxX) then
			maxX = v[i][1]
		end

		if (v[i][2] < minY) then
			minY = v[i][2]
		elseif (v[i][2] > maxY) then
			maxY = v[i][2]
		end

	end

	return minX, minY, maxX - minX, maxY - minY

end

function col.getPolyCenter(v)

	local x, y, w, h = col.getPolyRect(v)

	return x + w / 2, y + h / 2

end

function col.collideRectRect(x1, y1, w1, h1, x2, y2, w2, h2)

	return x1 + w1 >= x2 and x1 <= x2 + w2 and y1 + h1 >= y2 and y1 <= y2 + h2

end

function col.collideLineLine(x1, y1, x2, y2, x3, y3, x4, y4)

	local a = ((x4 - x3) * (y1 - y3) - (y4 - y3) * (x1 - x3)) / ((y4 - y3) * (x2 - x1) - (x4 - x3) * (y2 - y1));
	local b = ((x2 - x1) * (y1 - y3) - (y2 - y1) * (x1 - x3)) / ((y4 - y3) * (x2 - x1) - (x4 - x3) * (y2 - y1));

	if (a >= 0 and a <= 1 and b >= 0 and b <= 1) then

		local x = x1 + (a * (x2 - x1));
		local y = y1 + (a * (y2 - y1));

		return true, x, y

	end

	return false

end

function col.collideLinePoly(x1, y1, x2, y2, v)

	for i = 1, #v do

		local vc, vn = getPolyNextLine(v, i)

		if (col.collideLineLine(vc[1], vc[2], vn[1], vn[2], x1, y1, x2, y2)) then
			return true, {vc[1], vc[2], vn[1], vn[2]} , {x1, y1, x2, y2}
		end

	end

	return false

end

function col.collidePointPoly(x, y, v)

	local collision = false

	for i = 1, #v do

		local vc, vn = getPolyNextLine(v, i)

		if (((vc[2] > y and vn[2] < y) or (vc[2] < y and vn[2] > y)) and (x < (vn[1] - vc[1]) * (y - vc[2]) / (vn[2] - vc[2]) + vc[1])) then
			collision = not collision
		end

	end

	return collision

end

function col.collidePolyPoly(v1, v2)

	local x1, y1, w1, h1 = col.getPolyRect(v1)
	local x2, y2, w2, h2 = col.getPolyRect(v2)

	if (not col.collideRectRect(x1, y1, w1, h1, x2, y2, w2, h2)) then
		return false
	end

	for i = 1, #v1 do

		local vc, vn = getPolyNextLine(v1, i)
		local collision, l1, l2 = col.collideLinePoly(vc[1], vc[2], vn[1], vn[2], v2)

		if (collision) then
			return collision, l1, l2
		end

	end

	if (col.collidePointPoly(v1[1][1], v1[1][2], v2) or col.collidePointPoly(v2[1][1], v2[1][2], v1)) then
		return true
	end

	return false

end

function col.sat(v1, v2)

	local function getNormals(v)

		local normals = {}

		for i = 1, #v do

			local p1 = v[i]
			local p2 = v[i + 1] or v[1]
			local edgeX, edgeY = p1[1] - p2[1], p1[2] - p2[2]
			local normalX, normalY = edgeY, -edgeX
			local mag = math.sqrt(math.pow(normalX, 2) + math.pow(normalY, 2))
			local unitX, unitY = normalX / mag, normalY / mag

			normals[#normals + 1] = { unitX, unitY }

		end

		return normals

	end

	local function project(v, a)

		local min = v[1][1] * a[1] + v[1][2] * a[2]
		local max = min

		for i = 2, #v do

			local proj = v[i][1] * a[1] + v[i][2] * a[2]

			if (proj < min) then
				min = proj
			elseif (proj > max) then
				max = proj
			end

		end

		return min, max

	end

	local axes = _.concat(getNormals(v1), getNormals(v2))
	local overlap = 9999999999
	local penetration = nil

	for i = 1, #axes do

		local axis = axes[i]
		local p1min, p1max = project(v1, axis)
		local p2min, p2max = project(v2, axis)

		if (p1min > p2max or p2min > p1max) then
			return false
		end

		local o = math.min(p1max, p2max) - math.max(p1min, p2min)

		if (o < overlap) then
			overlap = o
			penetration = axis
		end

	end

	return true, overlap, penetration

end

return col
