-- wengwengweng

local debug = {}

debug.ing = false
debug.ignores = 0
debug.messages = {}
debug.items = {}

function debug:toggle(d)
	self.ing = (d == nil and (not self.ing) or d)
end

function debug:log(m)
	self.messages[#self.messages + 1] = m
end

function debug:clear()
	self.messages = {}
end

function debug:ignoreDraw(s)
	self.ignores = self.ignores + (s or 1)
end

function debug:addItem(t)
	self.items[t] = {}
end

function debug:updateItem(t, i, n)
	self.items[t][i] = n
end

function debug:draw()

	if (not self.ing) then
		return
	end

	local icon = love.graphics.newImage(love.window.getIcon())
	local fps = love.timer.getFPS()
	local drawcalls = love.graphics.getStats().drawcalls
	local canvasswitches = love.graphics.getStats().canvasswitches
	local mouseX, mouseY = love.mouse.getPosition()
	local w, h = love.graphics.getDimensions()

	-- Back
	love.graphics.setColor({0, 0, 0, 0.6})
	love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), 112)
	love.graphics.setScissor(0, 0, love.graphics.getWidth(), 112)

	-- Game
	love.graphics.push()
	love.graphics.translate(32, 12)
	love.graphics.setColor({0, 0, 1, 1})
	love.graphics.print("GAME", 0, 0)
	love.graphics.setColor({1, 1, 1, 1})
	love.graphics.draw(icon, 0, 24, 0, 42 / icon:getWidth(), 42 / icon:getHeight())
	love.graphics.print(love.window.getTitle(), 0, 72)
	love.graphics.pop()

	-- Info
	love.graphics.push()
	love.graphics.translate(144, 12)
	love.graphics.setColor({1, 1, 0, 1})
	love.graphics.print("INFO", 0, 0)
	love.graphics.setColor({1, 1, 1, 1})
	love.graphics.print("mouse", 0, 24)

	if (love.mouse.getRelativeMode()) then
		love.graphics.print("relative", 96, 24)
	else
		love.graphics.print(mouseX .. " : " .. mouseY, 96, 24)
	end

	love.graphics.print("window", 0, 40)
	love.graphics.print(w .. " x " .. h, 96, 40)
	love.graphics.pop()

	-- Stats
	love.graphics.push()
	love.graphics.translate(396, 12)
	love.graphics.setColor({1, 0, 0, 1})
	love.graphics.print("STATS", 0, 0)
	love.graphics.setColor({1, 1, 1, 1})
	love.graphics.print("fps", 0, 24)
	love.graphics.print(fps, 96, 24)
	love.graphics.print("canvas", 0, 40)
	love.graphics.print(canvasswitches, 96, 40)
	love.graphics.print("draw", 0, 56)
	love.graphics.print(drawcalls - self.ignores, 96, 56)
	love.graphics.pop()

	-- Log
	love.graphics.push()
	love.graphics.translate(580, 12)
	love.graphics.setColor({0, 1, 0, 1})
	love.graphics.print("LOG", 0, 0)
	love.graphics.setColor({1, 1, 1, 1})

	for i = #self.messages, 1, -1 do

		love.graphics.print(self.messages[i], 0, 24 + (#self.messages - i) * 16)

		if (i <= #self.messages - 3) then
			break
		end

	end

	love.graphics.pop()

	-- Items
	local i = 0

	for name, item in pairs(self.items) do

		i = i + 1
		love.graphics.push()
		love.graphics.translate(720 + 180 * (i - 1), 12)
		love.graphics.print(name, 0, 0)
		love.graphics.pop()

	end

	-- Reset
	self.ignores = 0
	love.graphics.setColor({1, 1, 1, 1})
	love.graphics.setScissor()

end

return debug
