-- wengwengweng

local Input = Object:extend()

function Input:new()

	self._events = {
		button = {},
		axis = {}
	}

end

function Input:update(dt)

	-- Buttons
	local activated = false

	for i, e in ipairs(self._events.button) do

		if (e.type == "key") then
			activated = love.keyboard.isDown(e.button)
		elseif (e.type == "mouse") then
			activated = love.mouse.isDown(e.button)
		elseif (e.type == "gamepad") then
			activated = love.joystick.getJoysticks()[e.stick]:isGamepadDown(e.button)
		end

		if (activated) then

			if (e.keep) then

				e.isDown = true
				e.pressed(e.args(dt))

			else

				if (not e.isDown) then

					e.isDown = true
					e.pressed(e.args(dt))

				end

			end

		else

			if (e.isDown) then

				e.isDown = false
				e.released(e.args(dt))

			end

		end

	end

	-- Axis
	for i, e in ipairs(self._events.axis) do
		if (e.check()) then
			e.moved(e.args(dt))
		end
	end

end

function Input:keyDown(key, cb)
	for i, k in ipairs(key) do
		self:_addButtonEvent("key", {
			button = k,
			pressed = cb,
			keep = true,
			args = function(dt)
				return dt
			end
		})
	end
end

function Input:keyPressed(key, cb)
	for i, k in ipairs(key) do
		self:_addButtonEvent("key", {
			button = k,
			pressed = cb,
			keep = false
		})
	end
end

function Input:keyReleased(key, cb)
	for i, k in ipairs(key) do
		self:_addButtonEvent("key", {
			button = k,
			released = cb,
			keep = false
		})
	end
end

function Input:mouseDown(key, cb)
	for i, k in ipairs(key) do
		self:_addButtonEvent("mouse", {
			button = k,
			pressed = cb,
			keep = true,
			args = function(dt)
				return dt, Input.mouse.x, Input.mouse.y
			end
		})
	end
end

function Input:mousePressed(key, cb)
	for i, k in ipairs(key) do
		self:_addButtonEvent("mouse", {
			button = k,
			pressed = cb,
			keep = false,
			args = function(dt)
				return Input.mouse.x, Input.mouse.y
			end
		})
	end
end

function Input:mouseReleased(key, cb)
	for i, k in ipairs(key) do
		self:_addButtonEvent("mouse", {
			button = k,
			released = cb,
			keep = false,
			args = function(dt)
				return Input.mouse.x, Input.mouse.y
			end
		})
	end
end

function Input:gamepadDown(stick, key, cb)
	for i, k in ipairs(key) do
		self:_addButtonEvent("gamepad", { stick = stick, button = k, pressed = cb, keep = true })
	end
end

function Input:gamepadPressed(stick, key, cb)
	for i, k in ipairs(key) do
		self:_addButtonEvent("gamepad", { stick = stick, button = k, pressed = cb, keep = false })
	end
end

function Input:gamepadReleased(stick, key, cb)
	for i, k in ipairs(key) do
		self:_addButtonEvent("gamepad", { stick = stick, button = k, released = cb, keep = false })
	end
end

function Input:gamepadAxisMoved(stick, axis, cb)
	self:_addAxisEvent("gamepad", {
		stick = stick,
		axis = axis,
		moved = cb,
		args = function(dt)
			return dt, love.joystick.getJoysticks()[stick]:getGamepadAxis(axis)
		end
	})
end

function Input:mouseMoved(cb)
	self:_addAxisEvent("mouse", {
		moved = cb,
		check = function()
			return Input.mouse.dx ~= 0 or Input.mouse.dy ~= 0
		end,
		args = function(dt)
			return Input.mouse.x, Input.mouse.y, Input.mouse.dx, Input.mouse.dy
		end
	})
end

function Input:_addButtonEvent(t, opt)

	opt.type = t
	opt.isDown = false
	opt.pressed = opt.pressed or function() end
	opt.released = opt.released or function() end
	opt.check = opt.check or function() return true end
	opt.args = opt.args or function() end
	self._events.button[#self._events.button + 1] = opt

end

function Input:_addAxisEvent(t, opt)

	opt.type = t
	opt.moved = opt.moved or function() end
	opt.check = opt.check or function() return true end
	opt.args = opt.args or function() end
	self._events.axis[#self._events.axis + 1] = opt

end

Input.mouse = {

	x = 0,
	y = 0,
	dx = 0,
	dy = 0

}

function Input.reset()

	Input.mouse.dx = 0
	Input.mouse.dy = 0

end

function love.mousemoved(x, y, dx, dy, istouch)

	Input.mouse.x = x
	Input.mouse.y = y
	Input.mouse.dx = dx
	Input.mouse.dy = dy

end

return Input
