-- wengwengweng

local State = Object:extend()

function State:new(...)

	self.list = {}
	self.current = nil
	self.disabled = false
	self:add(...)

end

function State:add(...)

	for i, s in ipairs({...}) do

		self.list[s] = {
			name = s,
			enter = function() end,
			leave = function() end,
			update = function() end
		}

	end

end

function State:on(s)

	assert(self.list[s], "not a state")

	return function(t)

		self.list[s].enter = t.enter or function() end
		self.list[s].update = t.update or function() end
		self.list[s].leave = t.leave or function() end

	end

end

function State:set(s)

	if (self.disabled) then
		return
	end

	local previous = self.current and self.current.name or nil

	if (self.current) then

		self.current.leave(s)

		if (self.current.name ~= s) then

			self.current = self.list[s]
			self.current.enter(previous)

		end

	else

		self.current = self.list[s]
		self.current.enter(previous)

	end

end

function State:is(...)

	for i, s in ipairs({...}) do
		if (s == self.current.name) then
			return true
		end
	end

	return false

end

function State:disable()

	self.disabled = true

end

function State:update(dt)

	self.current.update(dt)

end

return State
