-- wengwengweng

local _ = {}

function _.random(...)
	return love.math.random(...)
end

function _.prob(p)
	return _.random() <= p
end

function _.choice(arr)
	if (_.isarr(arr)) then
		return arr[_.random(#arr)]
	elseif (arr == nil) then
		return _.choice({true, false})
	end
end

function _.sign(x)
	if (type(x) == "number") then
		return x < 0 and -1 or 1
	elseif (type(x) == "boolean") then
		return x and 1 or -1
	end
end

function _.round(x, increment)
	if (increment) then
		return _.round(x / increment) * increment
	end
	return x >= 0 and math.floor(x + .5) or math.ceil(x - .5)
end

function _.clamp(x, min, max)
	return x < min and min or (x > max and max or x)
end

function _.lerp(a, b, speed)
	return a + (b - a) * _.clamp(speed, 0, 1)
end

function _.smooth(a, b, speed)
	local t = _.clamp(speed, 0, 1)
	local m = t * t * (3 - 2 * t)
	return a + (b - a) * m
end

function _.angle(x1, y1, x2, y2)
	return math.atan2(y2 - y1, x2 - x1)
end

function _.vector(angle, mag)
	return math.cos(angle) * mag, math.sin(angle) * mag
end

function _.distance(x1, y1, x2, y2, squared)

	local dx = x1 - x2
	local dy = y1 - y2
	return math.sqrt(dx * dx + dy * dy)

end

function _.isarr(x)

	return type(x) == "table" and x[1] ~= nil

end

function _.iter(t)

	if (_.isarr(t)) then
		return ipairs
	else
		return pairs
	end

end

function _.has(t, item)

	for k, v in _.iter(t)(t) do
		if (v == item) then
			return true
		end
	end

	return false

end

function _.push(t, ...)
	for i = 1, select("#", ...) do
		t[#t + 1] = select(i, ...)
	end
	return ...
end

function _.each(t, f)
	for k, v in _.iter(t)(t) do
		f(k, v)
	end
end

function _.remove(t, x)
	for i, v in _.iter(t)(t) do
		if (v == x) then
			if (_.isarr(t)) then
				table.remove(t, i)
				return x
			else
				t[i] = nil
				return x
			end
		end
	end
	return nil
end

function _.concat(...)
	local list = {}
	for i = 1, select("#", ...) do
		local t = select(i, ...)
		if (t ~= nil) then
			for i, v in _.iter(t)(t) do
				list[#list + 1] = v
			end
		end
	end
	return list
end

function _.clone(t)
	local nt = {}
	for k, v in pairs(t) do
		nt[k] = v
	end
	return nt
end

function _.glitch()
	local r = _.choice
	local c = { r({0, 1}), r({0, 1}), r({0, 1}) }
	if (c[1] + c[2] + c[3] == 0 or c[1] + c[2] + c[3] == 1 * 3) then
		return _.glitch()
	end
	return c
end

return _
