-- wengwengweng

local ffi = require("ffi")
local Vector = {}

setmetatable(Vector, Vector)

ffi.cdef[[
	typedef struct {
		float x;
		float y;
	} vec;
]]

function Vector:__call(x, y)
	return ffi.new("vec", (x or 0), (y or 0))
end

function Vector:__index(k)
	return rawget(Vector, k)
end

function Vector:set(x, y)

	if (_isVector(x)) then
		self.x, self.y = x.x, x.y
	elseif (type(x) == "number" and type(y) == "number") then
		self.x, self.y = x, y
	else
		error()
	end

end

function Vector:add(x, y)

	if (_isVector(x)) then
		return Vector(self.x + x.x, self.y + x.y)
	elseif (type(x) == "number" and type(y) == "number") then
		return Vector(self.x + x, self.y + y)
	else
		error()
	end

end

function Vector:sub(x, y)

	if (_isVector(x)) then
		return Vector(self.x - x.x, self.y - x.y)
	elseif (type(x) == "number" and type(y) == "number") then
		return Vector(self.x - x, self.y - y)
	else
		error()
	end

end

function Vector:mul(t)

	return Vector(self.x * t, self.y * t)

end

function Vector:div(t)

	return self:mul(1 / t)

end

function Vector:equals(other)
	assert(_isVector(other), "not a vector")
	return self.x == other.x and self.y == other.y
end

function Vector:dot(other)
	assert(_isVector(other), "not a vector")
	return self.x * other.x + self.y * other.y
end

function Vector:mag()
	return math.sqrt(math.pow(self.x, 2) + math.pow(self.y, 2))
end

function Vector:unit()

	local mag = self:mag()

	return Vector(self.x / mag, self.y / mag)

end

function Vector:angle()
	return math.deg(math.atan2(self.y, self.x))
end

function Vector:clone()
	return Vector(self.x, self.y)
end

function Vector:toAngle(angle)

	local mag = self:mag()

	return Vector(math.cos(math.rad(angle)) * mag, math.sin(math.rad(angle)) * mag)

end

function Vector:toMag(mag)

	local angle = self:angle()

	return Vector(math.cos(math.rad(angle)) * mag, math.sin(math.rad(angle)) * mag)

end

function Vector:unpack()
	return self.x, self.y
end

function Vector:__add(other)
	assert(_isVector(other), "not a vector")
	return self:add(other)
end

function Vector:__sub(other)
	assert(_isVector(other), "not a vector")
	return self:sub(other)
end

function Vector:__mul(t)
	if (_isVector(t)) then
		return self:dot(t)
	elseif (type(t) == "number") then
		return self:mul(t)
	else
		error()
	end
end

function Vector:__div(t)
	return self:div(t)
end

function Vector:__eq(other)
	return self:equals(other)
end

function Vector:__tostring()
	return self.x .. " " .. self.y
end

function _isVector(v)

	if (ffi.istype("vec", v)) then
		return true
	else
		if (type(v) == "table") then
			if (type(v.x) == "number" and type(v.y) == "number") then
				return true
			end
		end
	end

	return false

end

ffi.metatype("vec", Vector)

return Vector
