-- wengwengweng

local Shake = Node:extend()

Shake.name = "Shake"
Shake.index = -999

function Shake:init()

	self:n(T(0, 0))
	self:setSize(love.graphics.getDimensions())
	self:setSpeed(8)

	self.props = {

		shake = 0,
		rotate = 0,
		shear = { x = 0, y = 0 },
		scale = { x = 1, y = 1 },

	}

	self.finals = {

		shake = 0,
		rotate = 0,
		shear = { x = 0, y = 0 },
		scale = { x = 1, y = 1 },

	}

end

function Shake:setSize(w, h)

	self.width = w or love.graphics.getWidth()
	self.height = h or love.graphics.getHeight()

	return self

end

function Shake:setSpeed(s)

	self.speed = s or 8

	return self

end

function Shake:shake(s)

	self.props.shake = s

	return self

end

function Shake:shakeTo(s)

	self.finals.shake = s

	return self

end

function Shake:rotate(r)

	self.props.rotate = r

	return self

end

function Shake:rotateTo(r)

	self.finals.rotate = r

	return self

end

function Shake:scale(x, y)

	if (not y) then
		return self:scale(x, x)
	else
		self.props.scale.x = x
		self.props.scale.y = y
	end

	return self

end

function Shake:scaleTo(x, y)

	if (not y) then
		return self:scale(x, x)
	else
		self.finals.scale.x = x
		self.finals.scale.y = y
	end

	return self

end

function Shake:shear(x, y)

	self.props.shear.x = x
	self.props.shear.y = y

	return self

end

function Shake:shearTo(x, y)

	self.finals.shear.x = x
	self.finals.shear.y = y

	return self

end

function Shake:update(dt)

	self.props.shake = _.lerp(self.props.shake, self.finals.shake, self.speed * dt)
	self.props.rotate = _.lerp(self.props.rotate, self.finals.rotate, self.speed * dt)
	self.props.scale.x = _.lerp(self.props.scale.x, self.finals.scale.x, self.speed * dt)
	self.props.scale.y = _.lerp(self.props.scale.y, self.finals.scale.y, self.speed * dt)
	self.props.shear.x = _.lerp(self.props.shear.x, self.finals.shear.x, self.speed * dt)
	self.props.shear.y = _.lerp(self.props.shear.y, self.finals.shear.y, self.speed * dt)

end

function Shake:draw()

	love.graphics.translate(self.width * .5 + self[T].pos.x, self.height * .5 + self[T].pos.y)
	love.graphics.rotate((math.random() - .5) * self.props.rotate)
	love.graphics.scale(self.props.scale.x, self.props.scale.y)
	love.graphics.translate(-(self.width * .5 + self[T].pos.x), -(self.height * .5 + self[T].pos.y))
	love.graphics.translate((math.random() - .5) * self.props.shake, (math.random() - .5) * self.props.shake)
	love.graphics.shear(self.props.shear.x * .01, self.props.shear.y * .01)

end

return Shake
