-- wengwengweng

local Sprite = Node:extend()

Sprite.name = "Sprite"

function Sprite:init(img, d)

	self:n(T(0, 0))
	self:n(Timer())
	self:setImg(img, d)
	self:setScale(1)
	self:setAngle(0)
	self:setOffset(0, 0)
	self:setShear(0, 0)
	self:setFlipping(false, false)

end

function Sprite:clip(name, bound, loop)

	assert(type(bound[1]) == "number" and bound[1] >= 1, "not valid bound")
	assert(type(bound[2]) == "number" and bound[2] <= #self.framelist, "not valid bound")

	self.anims[name] = {
		name = name,
		bound = bound,
		loop = (loop or false),
		on = function() end,
		complete = function() end,
		leave = function() end
	}

end

function Sprite:play(anim)

	assert(self.anims[anim], "not an clips")

	if (self.state) then
		self.state.leave()
	end

	self.state = self.anims[anim]
	self.state.on()
	self:start()
	self:goto(self.state.bound[1])
	self[Timer]:clear()

	self[Timer]:after(self.speed, function(t)

		if (self.frame >= self.state.bound[2]) then

			self.state.complete()

			if (self.state.loop) then
				self:goto(self.state.bound[1])
				t:reset()
			end

			return

		end

		self.frame = self.frame + 1
		t:reset()

	end)

end

function Sprite:is(t)
	return self.state.name == t
end

function Sprite:on(name, f)

	assert(self.anims[name], "not an anim")
	self.anims[name].on = f

end

function Sprite:leave(name, f)

	assert(self.anims[name], "not an anim")
	self.anims[name].leave = f

end

function Sprite:complete(name, f)

	assert(self.anims[name], "not an anim")
	self.anims[name].complete = f

end

function Sprite:setImg(img, d)

	assert(type(img) == "userdata", "img expected")
	self.img = img
	self.anims = {}
	self.framelist = {}
	self.state = nil
	self.frame = 1
	self.bound = { 1, 1 }
	self.speed = 0.1
	self.stopped = false
	self[Timer]:clear()

	local w, h = self.img:getWidth() / (d or 1), self.img:getHeight()

	for i = 1, (d or 1) do
		self.framelist[i] = love.graphics.newQuad((i - 1) * w, 0, w, h, w * (d or 1), h)
	end

	return self

end

function Sprite:setSpeed(s)

	self.speed = s

	return self

end

function Sprite:goto(f)

	self.frame = f

	return self

end

function Sprite:start()

	self[Timer]:setUpdate(true)

	return self

end

function Sprite:stop()

	self[Timer]:setUpdate(false)

	return self

end

function Sprite:setFlipping(x, y)

	self.flipping = {

		x = (x or false),
		y = (y or false)

	}

	return self

end

function Sprite:setQuad(x, y, w, h)

	assert(#self.framelist == 1, "can't set quad for animation")
	self.framelist[1]:setViewport(x, y, w, h)

	return self

end

function Sprite:setOffset(x, y)

	assert(type(x) == "number", "number expected")
	assert(type(y) == "number", "number expected")

	self.offset = {
		x = x,
		y = y
	}

	return self

end

function Sprite:setShear(x, y)

	assert(type(x) == "number", "number expected")
	assert(type(y) == "number", "number expected")

	self.shear = {
		x = x,
		y = y
	}

	return self

end

function Sprite:setScale(x, y)

	assert(type(x) == "number", "number expected")

	if (not y) then
		self.scale = { x = x, y = x }
	else
		self.scale = { x = x, y = y }
	end

	return self

end

function Sprite:setAngle(a)

	assert(type(a) == "number", "number expected")
	self.angle = a

	return self

end

function Sprite:getWidth()
	return self.scale.x * self.img:getWidth()
end

function Sprite:getHeight()
	return self.scale.y * self.img:getHeight()
end

function Sprite:getSize()
	return self:getWidth(), self:getHeight()
end

function Sprite:setSize(w, h)

	assert(type(w) == "number", "number expected")
	assert(type(h) == "number", "number expected")

	self.scale.x = w / self.img:getWidth() * #self.framelist
	self.scale.y = h / self.img:getHeight()

	return self

end

function Sprite:setWidth(w)

	local s = w / self:getWidth() * #self.framelist

	self.scale.x = self.scale.x * s
	self.scale.y = self.scale.y * s

	return self

end

function Sprite:setHeight(h)

	local s = h / self:getHeight()

	self.scale.x = self.scale.x * s
	self.scale.y = self.scale.y * s

	return self

end

function Sprite:draw()

	love.graphics.draw(

		self.img,
		self.framelist[self.frame],
		self[T].pos.x,
		self[T].pos.y,
		math.rad(self.angle) * _.sign(self.flipping.x) * _.sign(self.flipping.y),
		self.scale.x * _.sign(not self.flipping.x),
		self.scale.y * _.sign(not self.flipping.y),
		self.offset.x * self.img:getWidth() / #self.framelist,
		self.offset.y * self.img:getHeight(),
		self.shear.x,
		self.shear.y

	)

end

function Sprite:collect()

	self.img:release()

	for i, f in ipairs(self.framelist) do
		f:release()
	end

end

return Sprite
