-- wengwengweng

local Timer = Node:extend()
local Event = Object:extend()

Timer.name = "Timer"

function Event:new(t, f)

	assert(type(t) == "number", "number expected")
	assert(type(f) == "function", "function expected")

	self.counter = 0
	self.time = t
	self.callback = f
	self.done = false

end

function Event:update(dt)

	if (self.done) then
		return
	end

	self.counter = self.counter + dt

	if (self.counter >= self.time) then

		self.done = true
		self.callback(self)

	end

end

function Event:setTime(t)

	self.time = t

	return self

end

function Event:reset()

	self.counter = 0
	self.done = false

	return self

end

function Timer:init()

	self._events = {}

end

function Timer:after(t, f)

	_.push(self._events, Event(t, f))

end

function Timer:clear()

	self._events = {}

end

function Timer:script(f)

	local co = coroutine.wrap(f)

	co(function(t)

		self:after(t, co)
		coroutine.yield()

	end)

end

function Timer:update(dt)

	for i, e in ipairs(self._events) do

		if (e.done) then
			_.remove(self._events, e)
		end

		e:update(dt)

	end

end

return Timer
