-- wengwengweng

local ToothBrush = Item:extend()

ToothBrush.name = "ToothBrush"

function ToothBrush:init(x, y)

	-- Nodes
	self:n(T(x, y))
	self:n(Motion())
	self:n(Shake())
	self[Motion]:setSpeed(9)
	self:n(Sprite(assets.sprites.toothbrush, 2))
	self[Sprite]:setWidth(64):setOffset(0.5, 0.5)
	self:n(Box({ {-36, -16}, {36, -16}, {36, 16}, {-36, 16} }, self.parent[World]))
	self:n(Timer())

	-- State
	self.state = State("RAW", "PASTED", "ICED", "DEAD")
	self.state:set("RAW")

end

function ToothBrush:hover()

	if (self.parent == self.root) then
		return
	end

	cursor:set("hand")

end

function ToothBrush:click(index, x, y)

	if (Object.is(self.parent, Bedroom)) then

		self:to(self.root[Bag])
		sounds:get("pick"):setPitch(1)
		sounds:play("pick")

	elseif (Object.is(self.parent, Bag)) then

		if (self.parent:full()) then

			self:to(self.root)
			self[T].pos:set(x, y)
			cursor:set("normal")

		end

	elseif (Object.is(self.parent, Search)) then

		if (not screen:isMobile()) then

			local s = self.root[Scene]:current()

			if (Object.is(s, Frozen)) then

				local spot = s[Spot][1]

				if (spot:query(x, y)) then
					self:to(s)
				else
					self:to(self.root[Bag])
				end

			elseif (Object.is(s, Balcony)) then

				local paste = s[Paste]

				if (paste:query(x, y)) then

					if (paste:accept()) then

						self.root:to(Good)
						self.state:set("PASTED")

						self[Timer]:after(3, function()
							self:to(self.root[Bag])
						end)

					end

				else

					self:to(self.root[Bag])

				end

			elseif (Object.is(s, Yo)) then

				local eye = s[Eye]

				if (eye:query(x, y)) then

					if (self.state:is("ICED")) then

						self.state:set("DEAD")
						self[T].pos:set(760, 492)
						self:transform(7)
						self[Shake]:shakeTo(36)
						self[Motion]:setSpeed(24)
						sounds:play("drill")

						self[Timer]:script(function(wait)

							wait(0.5)
							self[Motion]:to(480, 492)
							wait(0.5)
							self[Motion]:to(760, 492)
							wait(0.5)
							self[Motion]:to(480, 492)
							wait(0.5)
							self[Motion]:to(760, 492)
							wait(0.5)
							self[Motion]:to(480, 492)
							wait(0.5)
							eye:clean()
							self.parent:destroy(self)

						end)

						return

					end

				end

				self:to(self.root[Bag])

			else

				self:to(self.root[Bag])

			end

		end

	elseif (Object.is(self.parent, Frozen)) then

		if (self.parent:full()) then

			self:to(self.root[Bag])
			sounds:get("pick"):setPitch(1)
			sounds:play("pick")

		end

	end

end

function ToothBrush:release(index, x, y)

	if (screen:isMobile()) then

		local s = self.root[Scene]:current()

		if (Object.is(s, Frozen)) then

			local spot = s[Spot][1]

			if (spot:query(x, y)) then
				self:to(s)
			else
				self:to(self.root[Bag])
			end

		elseif (Object.is(s, Balcony)) then

			local paste = s[Paste]

			if (paste:query(x, y)) then

				if (paste:accept()) then

					self.root:to(Good)
					self.state:set("PASTED")

					self[Timer]:after(3, function()
						self:to(self.root[Bag])
					end)

				end

			else

				self:to(self.root[Bag])

			end

		elseif (Object.is(s, Yo)) then

			local eye = s[Eye]

			if (eye:query(x, y)) then

				if (self.state:is("ICED")) then

					self.state:set("DEAD")
					self[T].pos:set(760, 492)
					self:transform(7)
					self[Shake]:shakeTo(36)
					self[Motion]:setSpeed(24)
					sounds:play("drill")

					self[Timer]:script(function(wait)

						wait(0.5)
						self[Motion]:to(480, 492)
						wait(0.5)
						self[Motion]:to(760, 492)
						wait(0.5)
						self[Motion]:to(480, 492)
						wait(0.5)
						self[Motion]:to(760, 492)
						wait(0.5)
						self[Motion]:to(480, 492)
						wait(0.5)
						eye:clean()
						self.parent:destroy(self)

					end)

					return

				end

			end

			self:to(self.root[Bag])

		else

			self:to(self.root[Bag])

		end

	end

end

function ToothBrush:to(s)

	s:fetch(self)
	self[Box][1]:setWorld(self.parent[World])
	self:change()

end

function ToothBrush:change()

	local index

	if (Object.is(self.parent, Frozen)) then
		index = 4
	else
		index = 1
	end

	if (self.state:is("RAW")) then
		self:transform(index)
	elseif (self.state:is("PASTED")) then
		self:transform(index + 1)
	elseif (self.state:is("ICED")) then
		self[Shake]:shakeTo(6)
		self:transform(index + 2)
	end

end

function ToothBrush:transform(i)

	local map = {

		{
			sprite = assets.sprites.toothbrush,
			n = 2,
			bound = {1, 1},
			width = 64,
			box = { {-36, -16}, {36, -16}, {36, 16}, {-36, 16} }
		},

		{
			sprite = assets.sprites.toothbrush,
			n = 2,
			bound = {2, 2},
			width = 64,
			box = { {-36, -16}, {36, -16}, {36, 16}, {-36, 16} }
		},

		{
			sprite = assets.sprites.toothbrush,
			n = 2,
			bound = {2, 2},
			width = 64,
			box = { {-36, -16}, {36, -16}, {36, 16}, {-36, 16} }
		},

		{
			sprite = assets.sprites.toothbrush2,
			n = 4,
			bound = {1, 1},
			width = 144,
			box = { {-72, -92}, {72, -92}, {72, 92}, {-72, 92} }
		},

		{
			sprite = assets.sprites.toothbrush2,
			n = 4,
			bound = {2, 2},
			width = 144,
			box = { {-72, -92}, {72, -92}, {72, 92}, {-72, 92} }
		},

		{
			sprite = assets.sprites.toothbrush2,
			n = 4,
			bound = {2, 4},
			width = 144,
			box = { {-72, -92}, {72, -92}, {72, 92}, {-72, 92} }
		},

		{
			sprite = assets.sprites.toothbrush3,
			n = 1,
			bound = {1, 1},
			width = 320,
			box = { {-72, -92}, {72, -92}, {72, 92}, {-72, 92} }
		}

	}

	local data = map[i]

	self[Sprite]:setImg(data.sprite, data.n):setWidth(data.width)
	self[Sprite]:clip("anim", data.bound, true)
	self[Sprite]:play("anim")
	self[Box][1]:setVertices(data.box)

end

return ToothBrush
